/*
 * @copyright		© Copyright 2006-2012, epages GmbH, All Rights Reserved.
 *
 * @module			ep.ui.slides
 *
 * @revision		$Revision: 1.1.2.1.2.1 $
 */

jQuery.plugin( "ep.ui.slides", {
	plugin:	["ep.ui.core", "jQuery.event.special.load", "jQuery.event.special.touch", "jQuery.fn.class", "jQuery.tmpl", "jQuery.ui.widget", "ep.dict"],
  dict: ["ep.dict"],
	tmpl:	["ep.ui.slides"]
}, function( $, undefined ){

	var fx = ep.ui.changeOverFx,
    tExpand = ep.dict("ep.dict").translate("Expand");


	$.widget( "ep.ui.Slides", {
		options: {
		//	action			: "lightbox",		//
			images			: "img",			// selector / array of data / data / image node / array of image nodes / jQuery object
			effect			: "standard",		// change-over effect
			delay			: 3000,				// how long are the images visible in play mode
			autoplay		: false,			// flag whether to change images automatically by delay
			controls		: 'auto',			// show controls or not ('auto' / 'visible' / 'hidden')
			caption			: false,			// show caption of images
			count			: false,				// show count of images like (i of n)
			defaultSize: 'M'						// which image size should be used as default ( 'XS' / 'S' / 'M' / 'L' )
		},

		_create: function(){
			var self = this,
				o = self.options,
				imgs = self.images = ep.ui.imgData(o.images)
					.on("change.uiSlides", $.proxy(self, "_change") );
			self.animating = false;
			self.refresh = false;

			// load template and add element names to instance
			$.extend( self, $.tmpl( 'ep.ui.slides', [{expand:tExpand}] ).appendTo(self.element).tmplItem('elements') );

			self.ctrlPrevHandle
				.on("click.uiSlides", function(){
					imgs.current("prev");
				});

			self.ctrlNextHandle
				.on("click.uiSlides", function(){
					imgs.current("next");
				});

			self.container
				.on("click.uiSlides", function(event){
					if( !$(event.originalTarget).is('.ep-uiSlides-ctrl a') && o.action ){
						self._external(true);
						imgs.trigger(o.action);
					}
				})
				.on("touchdrag.uiSlides", function(event){
					event.preventDefault();
				})
				.on("touchswiperight.uiSlides", function(event){
					imgs.current("prev");
				})
				.on("touchswipeleft.uiSlides", function(event){
					imgs.current("next");
				});

			self._setOptions(o);
		},

		_init: function(){
			var self = this;

			self._internal(true);
			self._change();
		},

		destroy: function(){
			var self = this;

			self.images.off(".uiSlides");
			self.container.remove();
			self.Inherited(arguments);
		},

		_internal: function( set ){
			return arguments.length ? (this.eventInernal = set) : this.eventInernal;
		},

		_external: function( set ){
			var self = this;

			if( set ){
				self.eventExternal = true;
				self._autoplay();
			}

			return self.eventExternal;
		},

		_autoplay: function( set ){
			var self = this,
				o = self.options;

			clearInterval(self.playTimeout);

			if( o.autoplay && !self._external() ){
				// auto play with delay interval
				self.playTimeout = setTimeout(function(){
					self._internal(true);
					self.images.current("next");
				},o.delay);
			}
		},

		_setOption: function( key, value ) {
			var self = this;
			if ( key === "action" ) {
				self.container.css("cursor", value ? "pointer" : "");
			}
			else if( key === "controls" ){
				self.container
					.removeClass(/ep-uiSlides-ctrl-[\w]+/)
					.addClass("ep-uiSlides-ctrl-"+value);
			}

			return this.Inherited(arguments);
		},

		_change: function( internal ){
			var self	= this,
				o		= self.options;

			self._external( !self._internal() );
			self._internal( true );

			if( self.animating ){
				self.refresh = true;
			}
			else{
				self.refresh = false;
				self.animating = true;

				var	imgs		= self.images,
					imgsNumber	= imgs.currentItem + 1,
					imgsLength	= imgs.length,
					prev		= imgs.prev(),
					curr		= imgs.current() || {},
					next		= imgs.next(),

					boxWidth	= self.box.innerWidth(),
					boxHeight	= self.box.innerHeight(),

					imgSrc = curr['src'+ o.defaultSize],
					img,
					check,
					doCheck = false;

				if( boxWidth > 200 || boxHeight > 200 ){
					check = curr.srcL;
					doCheck = true;
				}
				else if( (boxWidth > 100 && boxWidth < 200) || (boxHeight > 100 && boxHeight < 200) ){
					check = curr.srcS;
					doCheck = true;
				}
				else if( boxWidth <= 100 || boxHeight <= 100 ){
					check = curr.srcXS;
					doCheck = true;
				}

				if(doCheck){
					var checkName = check.replace(/(_\w)?\..*?$/ig,"");
					var imgName = imgSrc.replace(/(_\w)?\..*?$/ig,"");
					if(checkName && imgName && checkName == imgName){
						imgSrc	= check;
					}
				}

				self.ctrlPrev.addClass('ui-hidden');
				self.ctrlNext.addClass('ui-hidden');

				// change curr <> next
				var boxNext = self.boxNext;
				self.boxNext = self.boxCurr;
				self.boxCurr = boxNext;

				self.desc
					.stop(true,true)
					.slideUp();

				img = $('<img alt="" src="'+imgSrc+'"/>')
					.on("load", function(){
						// call chang-over effect
						(fx(o.effect))(
							self.box,
							self.boxNext.stop(true,true),
							self.boxCurr.stop(true,true).hide().empty().append(img),
							o.delay,
							function(){
								self.ctrlPrev
									.toggleClass('ui-hidden',!prev);

								self.ctrlNext
									.toggleClass('ui-hidden',!next);

								self.descCount
									.html(imgsNumber+"/"+imgsLength)
									.toggle( !!o.count );

								self.descCaption
									.html(curr.desc.replace(/\n/g,"<br />"))
									.toggle( !!(o.caption && curr.desc) );

								if( o.count || (o.caption && curr.desc) ){
									self.desc
										.stop(true,true)
										.slideDown();
								}

								self.animating = false;

								if( self.refresh ){
									self._change();
								}
								else{
									self._autoplay();
								}
							}
						);
					})
					.appendTo('head');
			}
		}
	});

});